Attribute VB_Name = "modCache"
Option Explicit

' This program automates different actions within Outwar, an RPG-game.
'Copyright (C) 2004  Kristian S.Stangeland
'
'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; either version 2
'of the License, or (at your option) any later version.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Type TheData
    Ranking As Double
    PlayerName As String * 25
    Link As Double
    Power_Rating As Double
    Money As Double
    Bank As Double
End Type

Type Page
    TimeAdded As Date
    Number As Integer
    Type As Long
    TheData(99) As TheData
End Type

Public Const CacheVer = 2

Global SavedPages() As Page
Dim lFile As String, Tell%

Public Sub LoadDatabase(File As String)

' *** START ERROR HANDLER ***
On Error GoTo errHandler
If Err.Number <> 0 Then
errHandler: ErrorIn "LoadDatabase(File)", Array(File), EA_NORERAISE: HandleError: Exit Sub
End If
' *** BEGIN CODE ***

On Error GoTo FileError

Dim Ver&, B&

lFile = File
If Dir(File) = "" Then Tell = -1: Exit Sub

B = FreeFile

' Get the index
Open File For Binary As B
    Get B, , Ver
    If Ver <> CacheVer Then MsgBox "File has a wrong cache version", vbCritical, "File": GoTo FileError
    
    Get B, , Tell
    ReDim SavedPages(Tell)
    Get B, , SavedPages
Close B

Exit Sub
FileError:

Reset
If Dir(File) <> "" Then Kill File
Tell = -1

End Sub

Public Sub SaveDatabase()

' *** START ERROR HANDLER ***
On Error GoTo errHandler
If Err.Number <> 0 Then
errHandler: ErrorIn "SaveDatabase()", Array(), EA_NORERAISE: HandleError: Exit Sub
End If
' *** BEGIN CODE ***

Dim Ver&, B&

' Cacheversion
Ver = CacheVer

If Dir(lFile) <> "" And Not lFile = "" Then Kill lFile

B = FreeFile

Open lFile For Binary As B
    Put B, , Ver
    Put B, , Tell
    Put B, , SavedPages
Close B

End Sub

Public Function FindPage(ByVal Number As Integer, lType As Long) As Long

' *** START ERROR HANDLER ***
On Error GoTo errHandler
If Err.Number <> 0 Then
errHandler: ErrorIn "FindPage(Number, lType)", Array(Number, lType), EA_NORERAISE: HandleError: Exit Function
End If
' *** BEGIN CODE ***

Dim A&

If Tell < 0 Then FindPage = -1: Exit Function

For A = LBound(SavedPages) To UBound(SavedPages)
    If SavedPages(A).Number = Number And SavedPages(A).Type = lType Then FindPage = A: Exit Function
Next

FindPage = -1

End Function

Public Sub AddPage(Data() As TheData, ByVal Number As Integer, lType As Long)

' *** START ERROR HANDLER ***
On Error GoTo errHandler
If Err.Number <> 0 Then
errHandler: ErrorIn "AddPage(Number, lType)", Array(Number, lType), EA_NORERAISE: HandleError: Exit Sub
End If
' *** BEGIN CODE ***

Tell = Tell + 1
ReDim Preserve SavedPages(Tell)

SetPage Tell, Data, Number, lType

End Sub

Public Sub SetPage(ByVal Index As Integer, Data() As TheData, ByVal Number As Integer, lType As Long)

' *** START ERROR HANDLER ***
On Error GoTo errHandler
If Err.Number <> 0 Then
errHandler: ErrorIn "SetPage(Index, Number, lType)", Array(Index, Number, lType), EA_NORERAISE: HandleError: Exit Sub
End If
' *** BEGIN CODE ***

Dim A&, B&, Lenght&

SavedPages(Index).Type = lType
SavedPages(Index).Number = Number
SavedPages(Index).TimeAdded = Now

For A = 0 To 99
    SavedPages(Index).TheData(A).Bank = Data(A).Bank
    SavedPages(Index).TheData(A).Link = Data(A).Link
    SavedPages(Index).TheData(A).Money = Data(A).Money
    SavedPages(Index).TheData(A).PlayerName = Data(A).PlayerName
    SavedPages(Index).TheData(A).Power_Rating = Data(A).Power_Rating
    SavedPages(Index).TheData(A).Ranking = Data(A).Ranking
Next

End Sub

Public Sub GetData(ByVal Index As Integer, Data() As TheData)

' *** START ERROR HANDLER ***
On Error GoTo errHandler
If Err.Number <> 0 Then
errHandler: ErrorIn "GetData(Index)", Array(Index), EA_NORERAISE: HandleError: Exit Sub
End If
' *** BEGIN CODE ***

Dim A&

For A = 0 To 99
    Data(A).Bank = SavedPages(Index).TheData(A).Bank
    Data(A).Link = SavedPages(Index).TheData(A).Link
    Data(A).Money = SavedPages(Index).TheData(A).Money
    Data(A).PlayerName = SavedPages(Index).TheData(A).PlayerName
    Data(A).Power_Rating = SavedPages(Index).TheData(A).Power_Rating
    Data(A).Ranking = SavedPages(Index).TheData(A).Ranking
Next

End Sub

Public Function GetTimeAdded(ByVal Index As Integer) As Date

' *** START ERROR HANDLER ***
On Error GoTo errHandler
If Err.Number <> 0 Then
errHandler: ErrorIn "GetTimeAdded(Index)", Array(Index), EA_NORERAISE: HandleError: Exit Function
End If
' *** BEGIN CODE ***

If Index < 0 Then GetTimeAdded = Now: Exit Function
GetTimeAdded = SavedPages(Index).TimeAdded

End Function

Public Sub ResetDatabase()

' *** START ERROR HANDLER ***
On Error GoTo errHandler
If Err.Number <> 0 Then
errHandler: ErrorIn "ResetDatabase()", Array(), EA_NORERAISE: HandleError: Exit Sub
End If
' *** BEGIN CODE ***

If Not Dir(lFile) = "" Then Kill lFile
Tell = -1
Erase SavedPages

End Sub

Public Function GetSock(ByVal Ind As Integer) As Long

' *** START ERROR HANDLER ***
On Error GoTo errHandler
If Err.Number <> 0 Then
errHandler: ErrorIn "GetSock(Ind)", Array(Ind), EA_NORERAISE: HandleError: Exit Function
End If
' *** BEGIN CODE ***

Select Case Ind
    Case 0: GetSock = 1
    Case 1: GetSock = 10
    Case 2: GetSock = 25
    Case 3: GetSock = 50
    Case 4: GetSock = 100
    Case 5: GetSock = 200
    Case 6: GetSock = 300
    Case Else: GetSock = 1
End Select

End Function

